@@ -1,12 +1,22 @@
 #ifndef RUBY_SOCKET_H
 #define RUBY_SOCKET_H 1
 
-#include "ruby/ruby.h"
-#include "ruby/io.h"
-#include "ruby/thread.h"
-#include "ruby/util.h"
-#include "internal.h"
+#include "ruby/config.h"
+#include RUBY_EXTCONF_H
+
+#if defined(__sun) || defined(_AIX)
+/* (Recent?)  Solaris' <nfs/nfs.h> have conflicting definition of T_DATA.  Let
+ * us honour system definition by undefining ours.
+ *
+ * See also [ruby-core:4261]
+ */
+# include "ruby/ruby.h"
+# undef T_DATA
+#endif
+
+#include <errno.h>
 #include <stdio.h>
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -50,12 +60,11 @@
 #ifdef HAVE_NETPACKET_PACKET_H
 #  include <netpacket/packet.h>
 #endif
+
 #ifdef HAVE_NET_ETHERNET_H
 #  include <net/ethernet.h>
 #endif
 
-#include <errno.h>
-
 #ifdef HAVE_SYS_UN_H
 #  include <sys/un.h>
 #endif
@@ -81,12 +90,15 @@
 #  endif
 #  include <ifaddrs.h>
 #endif
+
 #ifdef HAVE_SYS_IOCTL_H
 #  include <sys/ioctl.h>
 #endif
+
 #ifdef HAVE_SYS_SOCKIO_H
 #  include <sys/sockio.h>
 #endif
+
 #ifdef HAVE_NET_IF_H
 #  include <net/if.h>
 #endif
@@ -94,16 +106,41 @@
 #ifdef HAVE_SYS_PARAM_H
 #  include <sys/param.h>
 #endif
+
 #ifdef HAVE_SYS_UCRED_H
 #  include <sys/ucred.h>
 #endif
+
 #ifdef HAVE_UCRED_H
 #  include <ucred.h>
 #endif
+
 #ifdef HAVE_NET_IF_DL_H
 #  include <net/if_dl.h>
 #endif
 
+#ifdef SOCKS5
+#  include <socks.h>
+#endif
+
+#ifndef HAVE_GETADDRINFO
+#  include "addrinfo.h"
+#endif
+
+#include "internal.h"
+#include "internal/array.h"
+#include "internal/error.h"
+#include "internal/gc.h"
+#include "internal/io.h"
+#include "internal/thread.h"
+#include "internal/vm.h"
+#include "ruby/io.h"
+#include "ruby/ruby.h"
+#include "ruby/thread.h"
+#include "ruby/util.h"
+#include "sockport.h"
+#include "ruby/fiber/scheduler.h"
+
 #ifndef HAVE_TYPE_SOCKLEN_T
 typedef int socklen_t;
 #endif
@@ -137,11 +174,6 @@ unsigned int if_nametoindex(const char *);
  */
 #define pseudo_AF_FTIP pseudo_AF_RTIP
 
-#ifndef HAVE_GETADDRINFO
-#  include "addrinfo.h"
-#endif
-
-#include "sockport.h"
 
 #ifndef NI_MAXHOST
 #  define NI_MAXHOST 1025
@@ -223,7 +255,6 @@ typedef union {
 #define INET_SOCKS  2
 
 extern int rsock_do_not_reverse_lookup;
-extern int rsock_cmsg_cloexec_state;
 #define FMODE_NOREVLOOKUP 0x100
 
 /* common socket families only */
@@ -249,9 +280,7 @@ extern VALUE rb_eSocket;
 
 #ifdef SOCKS
 extern VALUE rb_cSOCKSSocket;
-#  ifdef SOCKS5
-#    include <socks.h>
-#  else
+#  ifndef SOCKS5
 void SOCKSinit();
 int Rconnect();
 #  endif
@@ -285,13 +314,13 @@ struct rb_addrinfo {
   struct addrinfo *ai;
   int allocated_by_malloc;
 };
-int rb_getaddrinfo(const char *node, const char *service, const struct addrinfo *hints, struct rb_addrinfo **res);
 void rb_freeaddrinfo(struct rb_addrinfo *ai);
 VALUE rsock_freeaddrinfo(VALUE arg);
 int rb_getnameinfo(const struct sockaddr *sa, socklen_t salen, char *host, size_t hostlen, char *serv, size_t servlen, int flags);
 int rsock_fd_family(int fd);
 struct rb_addrinfo *rsock_addrinfo(VALUE host, VALUE port, int family, int socktype, int flags);
 struct rb_addrinfo *rsock_getaddrinfo(VALUE host, VALUE port, struct addrinfo *hints, int socktype_hack);
+
 VALUE rsock_fd_socket_addrinfo(int fd, struct sockaddr *addr, socklen_t len);
 VALUE rsock_io_socket_addrinfo(VALUE io, struct sockaddr *addr, socklen_t len);
 
@@ -317,7 +346,7 @@ int rsock_socket(int domain, int type, int proto);
 int rsock_detect_cloexec(int fd);
 VALUE rsock_init_sock(VALUE sock, int fd);
 VALUE rsock_sock_s_socketpair(int argc, VALUE *argv, VALUE klass);
-VALUE rsock_init_inetsock(VALUE sock, VALUE remote_host, VALUE remote_serv, VALUE local_host, VALUE local_serv, int type);
+VALUE rsock_init_inetsock(VALUE sock, VALUE remote_host, VALUE remote_serv, VALUE local_host, VALUE local_serv, int type, VALUE resolv_timeout, VALUE connect_timeout);
 VALUE rsock_init_unixsock(VALUE sock, VALUE path, int server);
 
 struct rsock_send_arg {
@@ -342,9 +371,9 @@ VALUE rsock_s_recvfrom_nonblock(VALUE sock, VALUE len, VALUE flg, VALUE str,
 			        VALUE ex, enum sock_recv_type from);
 VALUE rsock_s_recvfrom(VALUE sock, int argc, VALUE *argv, enum sock_recv_type from);
 
-int rsock_connect(int fd, const struct sockaddr *sockaddr, int len, int socks);
+int rsock_connect(int fd, const struct sockaddr *sockaddr, int len, int socks, struct timeval *timeout);
 
-VALUE rsock_s_accept(VALUE klass, int fd, struct sockaddr *sockaddr, socklen_t *len);
+VALUE rsock_s_accept(VALUE klass, VALUE io, struct sockaddr *sockaddr, socklen_t *len);
 VALUE rsock_s_accept_nonblock(VALUE klass, VALUE ex, rb_io_t *fptr,
 			      struct sockaddr *sockaddr, socklen_t *len);
 VALUE rsock_sock_listen(VALUE sock, VALUE log);
@@ -403,33 +432,25 @@ NORETURN(void rsock_sys_fail_sockaddr(const char *, struct sockaddr *addr, sockl
 NORETURN(void rsock_sys_fail_raddrinfo(const char *, VALUE rai));
 NORETURN(void rsock_sys_fail_raddrinfo_or_sockaddr(const char *, VALUE addr, VALUE rai));
 
-/*
- * It is safe on Linux to attempt using a socket without waiting on it in
- * all cases.  For some syscalls (e.g. accept/accept4), blocking on the
- * syscall instead of relying on select/poll allows the kernel to use
- * "wake-one" behavior and avoid the thundering herd problem.
- * This is likely safe on all other *nix-like systems, so this whitelist
- * can be expanded by interested parties.
- */
-#if defined(__linux__)
-static inline int rsock_maybe_fd_writable(int fd) { return 1; }
-static inline void rsock_maybe_wait_fd(int fd) { }
-#  ifdef MSG_DONTWAIT
-#    define MSG_DONTWAIT_RELIABLE 1
-#  endif
-#else /* some systems (mswin/mingw) need these.  ref: r36946 */
-#  define rsock_maybe_fd_writable(fd) rb_thread_fd_writable((fd))
-#  define rsock_maybe_wait_fd(fd) rb_thread_wait_fd((fd))
+#if defined(__MINGW32__) || defined(_WIN32)
+#define RSOCK_WAIT_BEFORE_BLOCKING
 #endif
 
 /*
  * some OSes may support MSG_DONTWAIT inconsistently depending on socket
  * type, we only expect Linux to support it consistently for all socket types.
  */
-#ifndef MSG_DONTWAIT_RELIABLE
+#if defined(MSG_DONTWAIT) && defined(__linux__)
+#  define MSG_DONTWAIT_RELIABLE 1
+#else
 #  define MSG_DONTWAIT_RELIABLE 0
 #endif
 
+VALUE rsock_read_nonblock(VALUE sock, VALUE length, VALUE buf, VALUE ex);
+VALUE rsock_write_nonblock(VALUE sock, VALUE buf, VALUE ex);
+
+void rsock_make_fd_nonblock(int fd);
+
 #if !defined HAVE_INET_NTOP && ! defined _WIN32
 const char *inet_ntop(int, const void *, char *, size_t);
 #elif defined __MINGW32__
