@@ -1,3 +1,4 @@
+/* -*- mode: c; indent-tabs-mode: t -*- */
 /**********************************************************************
 
   stringio.c -
@@ -11,7 +12,7 @@
 
 **********************************************************************/
 
-#define STRINGIO_VERSION "3.0.0"
+#define STRINGIO_VERSION "3.0.1"
 
 #include "ruby.h"
 #include "ruby/io.h"
@@ -64,7 +65,7 @@ strio_extract_modeenc(VALUE *vmode_p, VALUE *vperm_p, VALUE opthash,
 		n = strchr(n, '|');
 	    }
 	    e = strchr(++n, ':');
-	    len = e ? e - n : strlen(n);
+	    len = e ? e - n : (long)strlen(n);
 	    if (len > 0 && len <= ENCODING_MAXNAMELEN) {
 		if (e) {
 		    memcpy(encname, n, len);
@@ -599,6 +600,14 @@ strio_closed_write(VALUE self)
     return Qtrue;
 }
 
+static struct StringIO *
+strio_to_read(VALUE self)
+{
+    struct StringIO *ptr = readable(self);
+    if (ptr->pos < RSTRING_LEN(ptr->string)) return ptr;
+    return NULL;
+}
+
 /*
  * call-seq:
  *   strio.eof     -> true or false
@@ -610,8 +619,7 @@ strio_closed_write(VALUE self)
 static VALUE
 strio_eof(VALUE self)
 {
-    struct StringIO *ptr = readable(self);
-    if (ptr->pos < RSTRING_LEN(ptr->string)) return Qfalse;
+    if (strio_to_read(self)) return Qfalse;
     return Qtrue;
 }
 
@@ -821,11 +829,11 @@ strio_get_sync(VALUE self)
 static VALUE
 strio_each_byte(VALUE self)
 {
-    struct StringIO *ptr = readable(self);
+    struct StringIO *ptr;
 
     RETURN_ENUMERATOR(self, 0, 0);
 
-    while (ptr->pos < RSTRING_LEN(ptr->string)) {
+    while ((ptr = strio_to_read(self)) != NULL) {
 	char c = RSTRING_PTR(ptr->string)[ptr->pos++];
 	rb_yield(CHR2FIX(c));
     }
@@ -1064,11 +1072,7 @@ strio_each_codepoint(VALUE self)
 
     ptr = readable(self);
     enc = get_enc(ptr);
-    for (;;) {
-	if (ptr->pos >= RSTRING_LEN(ptr->string)) {
-	    return self;
-	}
-
+    while ((ptr = strio_to_read(self)) != NULL) {
 	c = rb_enc_codepoint_len(RSTRING_PTR(ptr->string)+ptr->pos,
 				 RSTRING_END(ptr->string), &n, enc);
 	ptr->pos += n;
@@ -1229,8 +1233,9 @@ strio_getline(struct getline_arg *arg, struct StringIO *ptr)
 	str = strio_substr(ptr, ptr->pos, e - s - w, enc);
     }
     else {
-	if (n < e - s) {
-	    if (e - s < 1024) {
+	if (n < e - s + arg->chomp) {
+	    /* unless chomping, RS at the end does not matter */
+	    if (e - s < 1024 || n == e - s) {
 		for (p = s; p + n <= e; ++p) {
 		    if (MEMCMP(p, RSTRING_PTR(str), char, n) == 0) {
 			e = p + (arg->chomp ? 0 : n);
